#ifndef GEO_NEXT_NODE_H_
#define GEO_NEXT_NODE_H_

#include <mobilenode.h>
#include <time.h> 
#include <random.h>
#include "geo_node.h"
#include "geo_utility.h"
#include <god.h>

// algorithm identification
#define CASUAL 		0
#define GREEDY 		1
#define COMPASS 	2
#define MFR 		3
#define ELLIPSOID 	4
#define PAB3D 		5
#define PROJECTIVE 	6
#define CFACE3 		7
#define ALSP 		8
#define GALSPG 		9
#define PFACEP 		10
#define PFACE3 		11
#define GPG	   		12
#define LAR 		13
#define PLAR		14
#define DFS			15
//define NEW			16

#define ABS 50		// ttl value for ALSP algorithm
#define LAR_R 250	// destination expected zone's radius for LAR algorithm
#define FAIL -1

class GEO_data;
class DFSAliveTimer : public TimerHandler {

public:
	DFSAliveTimer(int id, GEO_data * geo_d) : TimerHandler() {id_ = id; geo_data_ = geo_d;}
	DFSAliveTimer();
protected:
	int id_;
	GEO_data * geo_data_;
	virtual void expire(Event * e);
};

class DFS_record {
public:
	int id;
	nsaddr_t from;
	nsaddr_t active;
	GNeighbors keep_on;
	DFSAliveTimer dfs_timer_;
	std::vector<nsaddr_t> deleted;

	//DFS_record() {}
	DFS_record(int, nsaddr_t, nsaddr_t, GNeighbors, GEO_data *);
	inline int size() {
		return keep_on.size();
	}
};

// Node's memory space for routing data
class GEO_data {
public:
	//DFS protocol
	 DFS_record * dfs_data_[200];
	 int dfs_size;

	 GEO_data();
	 void dfs_add(int, nsaddr_t, nsaddr_t, GNeighbors);
	 void dfs_del(int);
	 DFS_record * dfs_get(int);
	 inline int size() { 
	 	/*int s = 0;
	 	for(int i = 0; i < dfs_size; i++)
	 		s += dfs_data[i].size();
	 	return s;*/
	 	return dfs_size;
	 }
};

class GeoNextNode {

public:
	GeoNextNode();
  	~GeoNextNode();
	GeoNextNode(int, nsaddr_t, int, int, int, int, int, int, Trace*, int);

	nsaddr_t nextNode(MobileNode*, Packet*, GNeighbors);
	nsaddr_t progress(int, bool, GNeighbors);
	std::vector<nsaddr_t> rw();
	void createProj(float, float, float, float, float, float, float, float, float, GNeighbors *);
	nsaddr_t findCloserOnFace(GNeighbors, nsaddr_t, float, float, float, float);
	nsaddr_t face2(float, float, float, float, float, float, float, float, float, bool);
	nsaddr_t dfs();
	void createGG(GNeighbors, GNeighbors *); 
	void setLSP();
	void setSDP();
	void setSDO();
	void setNextP();

private:
	int algo_;				// forward algorithm
	nsaddr_t addr_;			// node's address
	float x_, y_, z_;		// node's coordinates
	int M_, R_, S_, P_; 	// PAB3D parameters
	int THR_R_, THR_F_;		// Random and Face TTL threshold values
	Trace* logtargetnext_; 	// log trace
	int geo_trace_;	
	GEO_data geo_data_;

	GNeighbors nbr_;	// neighbors object

	struct hdr_geo* hdr;	// GEO header
	struct hdr_cmn* ch;		// COMMON header	

	void trace(char* fmt, ...);					// prototipe trace print
};

#endif




