#include "geo_node.h"
#include <cmath>
#include <iostream>
#include <string.h>
#include "geo_utility.h"

GNode::GNode(nsaddr_t id, float x, float y, float z, double ts, GNeighbors* nbr) : addr_(id), x_(x), y_(y), z_(z), ts_(ts), nat_(id, nbr) { }

// delete neighbor when NeighborAliveTimer exipred
void NeighborAliveTimer::expire(Event *e) {
	nbr_->delByAddr(id_);
}


GNeighbors::GNeighbors() {
	nbSize_ = 0;
}

GNeighbors::~GNeighbors() {
	//free(nbr_);
}

int GNeighbors::size() {
	return nbSize_;
}

void GNeighbors::print(char * out) {

	char s[255];
	strcpy(s, "[");
	for(int i = 0; i < nbSize_; i++) {
		if(i < nbSize_ - 1) {
			char ad[5];
			sprintf(ad, "%d", nbr_.at(i)->addr_);
			strcat(s, ad);
			strcat(s, " ");
		}
		else {
			char ad[5];
			sprintf(ad, "%d", nbr_.at(i)->addr_);
			strcat(s, ad);
		}
	}
	strcat(s, "]");
	for(int i = 0; i < 255; i++)
	{
		out[i] = s[i];
	}
}

bool GNeighbors::empty() {
	if(nbSize_ == 0) return true;
	return false;
}

// Add node in the neighbor list
// If the node is already present return false, otherwise return true 
bool GNeighbors::addNbr(nsaddr_t id, float x, float y, float z, float beacon_alive) {
	
	for(std::vector<GNode*>::iterator it = nbr_.begin(); it != nbr_.end(); it++) {
		// se presente aggiorno solo le coordinate
		if((*it)->addr_ == id) {
			(*it)->x_ = x;
			(*it)->y_ = y;
			(*it)->z_ = z;
			(*it)->ts_ = CURRENT;
			(*it)->nat_.force_cancel();
			(*it)->nat_.resched(beacon_alive);
			return false;
		}
	}
	// aggiungo il nodo dentro la lista
	GNode *nn = new GNode(id, x, y, z, CURRENT, this);
	nn->nat_.sched(beacon_alive);
	nbr_.push_back(nn);
	nbSize_++;
	return true;
}

void GNeighbors::addNode(GNode * node) {
	nbr_.push_back(node);
	nbSize_++;
}

nsaddr_t GNeighbors::delByAddr(nsaddr_t id) {
	 for (unsigned i=0; i<nbr_.size(); i++) {
		if(nbr_.at(i)->addr_ == id) {
			nbr_.erase(nbr_.begin()+i);
			nbSize_--;
			return id;
		}
	}
	return -1;
}

GNode* GNeighbors::get(int i) {
	return nbr_.at(i);
}

GNode* GNeighbors::getByAddr(nsaddr_t addr) {
	for (unsigned i=0; i<nbr_.size(); i++)
		if(nbr_.at(i)->addr_ == addr)
			return nbr_.at(i);
	return NULL;
}

GNode* GNeighbors::front() {
	return nbr_.at(0);
}

void GNeighbors::clear() {
		nbr_.clear();
		nbSize_ = 0;
}

GNode * GNode::project(float x1, float y1, float z1,
				     float x2, float y2, float z2, 
				     float x3, float y3, float z3) {
	//coefficienti del piano su cui giace il triangolo
	float a,b,c;//d;
	a=(y2-y1)*(z3-z1)-(z2-z1)*(y3-y1); //cout<<"a: "<<a<<endl;
	b=(z2-z1)*(x3-x1)-(x2-x1)*(z3-z1); //cout<<"b: "<<b<<endl;
	c=(x2-x1)*(y3-y1)-(y2-y1)*(x3-x1); //cout<<"c: "<<c<<endl;
	//d=-x1*a-y1*b-z1*c; //cout<<"d: "<<d<<endl;
	//versore normale
	float nx, ny, nz;
	nx=a/sqrt(a*a+b*b+c*c);
	ny=b/sqrt(a*a+b*b+c*c);
	nz=c/sqrt(a*a+b*b+c*c); //cout<<"normale: "<<nx<<endl;
	//introduco la costante di moltiplicazione
	float cm;
	//cm=(a*P.x+b*P.y+c*P.z+d)/(a*a+b*b+c*c); cout<<"cm: "<<cm<<endl;
	cm=(x_-x1)*nx+(y_-y1)*ny+(z_-z1)*nz;
	//calcolo le tre coordinate del punto di proiezione
	float xp,yp,zp;
	//x1=P.x-cm*a;
	//y1=P.y-cm*b;
	//z1=P.z-cm*c;
	xp=x_-cm*nx;
	yp=y_-cm*ny;
	zp=z_-cm*nz;
	//controllo che la proiezione appartenga al piano
	//cout<<"la proiezione appartiene al piano?"<<endl;
	//float temp=a*x1+b*y1+c*z1+d;
	//cout<<temp<<endl;
	float dxz = dist(xp, 0, zp, x1, 0, z1);
	if(xp <= x1)
		xp = x1 - dxz;
	else
		xp = x1 + dxz;
	float dyz = dist(yp, 0, zp, y1, 0, z1);
	if(yp <= y1)
		yp = y1 - dyz;
	else
		yp = y1 + dyz;

	//if(yp < 0) yp = abs(yp); // correzione non è giusta TODO
	GNode * projGNode = new GNode(addr_, xp, yp, 0, 0, NULL);
	return projGNode;
}

